﻿using AuditManager.Data;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.ViewModels;
using Syncfusion.ListView.XForms;
using Syncfusion.XForms.ComboBox;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Forms;
using SelectionChangedEventArgs = Syncfusion.XForms.ComboBox.SelectionChangedEventArgs;

namespace AuditManager.Views
{

    public partial class DettaglioPage : BasePage
    {
        #region Props
        public DettaglioViewModel Modello { get; set; }

        private NavigationParameterModel Parametro { get; set; }
        #endregion

        #region Ctors
        public DettaglioPage(NavigationParameterModel parametro) : base()
        {
            try
            {
                Parametro = parametro;
                InitializeComponent();
            }
            catch (Exception ex)
            {

                throw;
            }
        }
        #endregion

        #region Overrides
        protected override void OnAppearing()
        {
            base.OnAppearing();

            MessagingCenter.Subscribe<DettaglioViewModel>(this, Messages.OperationStarted.ToString(), (sender) =>
            {
                content.IsVisible = false;
                busyindicator.IsVisible = true;
                busyindicator.IsEnabled = true;
            });

            MessagingCenter.Subscribe<DettaglioViewModel, bool>(this, Messages.OperationFinished.ToString(), (sender, test) =>
            {
                content.IsVisible = true;
                busyindicator.IsVisible = false;
                busyindicator.IsEnabled = false;

                if (test)
                    HideAttachmentMenu();
            });

            MessagingCenter.Subscribe<DettaglioViewModel>(this, Messages.Reset.ToString(), (sender) =>
            {
                content.IsVisible = true;
                busyindicator.IsVisible = false;
                busyindicator.IsEnabled = false;

                SvuotaForm();
            });

            MessagingCenter.Subscribe<DettaglioViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString(), async (sender, alert) =>
            {
                await AlertVibrate(alert);
            });

            MessagingCenter.Subscribe<DettaglioViewModel, ConfirmModel>(this, Messages.DisplayAlertMessage.ToString(), async (sender, obj) =>
            {
                if (await App.ShowConfirmAlert(obj))
                {
                    Modello.SvuotaForm();
                    await scrollViewForm.ScrollToAsync(0, 0, true);
                }
                else
                    await TornaIndietro();
            });

            MessagingCenter.Subscribe<DettaglioViewModel, bool>(this, Messages.GoBack.ToString(), async (sender, test) =>
            {
                if (test)
                    await TornaIndietro();
                //else
                //    await GoToEdit();
            });

            MessagingCenter.Subscribe<DettaglioViewModel>(this, QuestionMessages.AggiungiAllegato.ToString(), (sender) =>
            {
                ShowAttachmentMenu();
            });

            MessagingCenter.Subscribe<DettaglioViewModel>(this, Messages.OpenPicker.ToString(), (sender) =>
            {
                picker.SelectedItem = Modello.DataPrevistaRisoluzioneAsDate;
                picker.IsOpen = !picker.IsOpen;
            });

            if (BindingContext == null)
            {
                Modello = new DettaglioViewModel(Parametro);
                BindingContext = Modello;
            }

            PosizionaTendine();

            if (Device.Idiom == TargetIdiom.Desktop || Device.Idiom == TargetIdiom.Tablet)
            {
                AbsoluteLayout.SetLayoutBounds(stackLayoutAllegatiMenu, new Rectangle(0, 1, 1, 130));

                listViewAllegatiMenu.LayoutManager = new LinearLayout();
                listViewAllegatiMenu.Orientation = Orientation.Horizontal;
            }
            else
            {
                AbsoluteLayout.SetLayoutBounds(stackLayoutAllegatiMenu, new Rectangle(0, 1, 1, 320));

                listViewAllegatiMenu.LayoutManager = new GridLayout() { SpanCount = 3 };
            }

            AbsoluteLayout.SetLayoutFlags(stackLayoutAllegatiMenu, AbsoluteLayoutFlags.PositionProportional | AbsoluteLayoutFlags.WidthProportional);

            comboAttributi1.DisplayMemberPath = nameof(RisposteDettaglioAttributo.DescRisposteDettaglioAttributo);
            comboAttributi2.DisplayMemberPath = nameof(RisposteDettaglioAttributo.DescRisposteDettaglioAttributo);
            comboAttributi3.DisplayMemberPath = nameof(RisposteDettaglioAttributo.DescRisposteDettaglioAttributo);
            comboAttributi4.DisplayMemberPath = nameof(Attributo4.DescAttributo);
            comboAttributi4Multi.DisplayMemberPath = nameof(Attributo4.DescAttributo);
            comboAttributi5.DisplayMemberPath = nameof(RisposteDettaglioAttributo.DescRisposteDettaglioAttributo);
            comboStati.DisplayMemberPath = nameof(ComboboxItem.Name);

            Modello.SetVisibilities();
            Modello.InitText();
            Modello.InitCombobox();
            Modello.InitRisoluzione();
            Modello.GetAllegati();
            Modello.GeneraBottoniAllegati();

            SetForm();

            Modello.IsDirty = false;
        }

        protected override void OnDisappearing()
        {
            comboAttributi1.SelectionChanged -= ComboBox_SelectionChanged;
            comboAttributi2.SelectionChanged -= ComboBox_SelectionChanged;
            comboAttributi3.SelectionChanged -= ComboBox_SelectionChanged;
            comboAttributi4.SelectionChanged -= ComboBox_SelectionChanged;
            comboAttributi4Multi.SelectionChanged -= SfComboBox_SelectionChanged;
            comboAttributi5.SelectionChanged -= ComboBox_SelectionChanged;

            MessagingCenter.Unsubscribe<DettaglioViewModel>(this, Messages.OpenPicker.ToString());
            MessagingCenter.Unsubscribe<DettaglioViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString());
            MessagingCenter.Unsubscribe<DettaglioViewModel, ConfirmModel>(this, Messages.DisplayAlertMessage.ToString());
            MessagingCenter.Unsubscribe<DettaglioViewModel, bool>(this, Messages.GoBack.ToString());

            MessagingCenter.Unsubscribe<DettaglioViewModel>(this, QuestionMessages.AggiungiAllegato.ToString());

            MessagingCenter.Unsubscribe<DettaglioViewModel>(this, Messages.OperationStarted.ToString());
            MessagingCenter.Unsubscribe<DettaglioViewModel>(this, Messages.OperationFinished.ToString());
            MessagingCenter.Unsubscribe<DettaglioViewModel>(this, Messages.Reset.ToString());

            Modello.OnDisappearing();
            base.OnDisappearing();
        }

        //protected override bool OnBackButtonPressed()
        //{
        //    ConfirmModel confirmModel = new ConfirmModel()
        //    {
        //        Title = Resx.AppResources.StringConfirmBackDetailTitle,
        //        Description = Resx.AppResources.StringConfirmBackDetailDesc,
        //        OkButton = Resx.AppResources.LabelSi,
        //        KoButton = Resx.AppResources.LabelNO
        //    };

        //    if (Modello.IsDirty)
        //    {
        //        Task<bool> action = App.ShowConfirmAlert(confirmModel);

        //        action.ContinueWith(async task =>
        //        {
        //            if (action.Result)
        //            {
        //                if (Parametro.DoRollback)
        //                {
        //                    Parametro.Domanda.Risposta = Parametro.OldValue;
        //                    App.Database.UpdateAuditDomandeRisposte(Parametro.Domanda);
        //                }

        //                //return base.OnBackButtonPressed();
        //                await Navigation.PopAsync();
        //                return false;
        //            }
        //            else
        //                return true;
        //        });

        //        return true;
        //    }
        //    else
        //    {
        //        if (Parametro.DoRollback)
        //        {
        //            Parametro.Domanda.Risposta = Parametro.OldValue;
        //            App.Database.UpdateAuditDomandeRisposte(Parametro.Domanda);
        //        }

        //        return base.OnBackButtonPressed();
        //    }
        //}
        #endregion

        #region Common Methods
        private void PosizionaTendine()
        {
            if (Device.Idiom == TargetIdiom.Phone)
            {
                AbsoluteLayout.SetLayoutBounds(panelAttributo1, new Rectangle(0, 0, 1, .5));
                AbsoluteLayout.SetLayoutFlags(panelAttributo1, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo2, new Rectangle(0, 1, 1, .5));
                AbsoluteLayout.SetLayoutFlags(panelAttributo2, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo3, new Rectangle(0, 0, 1, .5));
                AbsoluteLayout.SetLayoutFlags(panelAttributo3, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo4, new Rectangle(0, 1, 1, .5));
                AbsoluteLayout.SetLayoutFlags(panelAttributo4, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo5, new Rectangle(0, 0, 1, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo5, AbsoluteLayoutFlags.All);
            }
            else
            {
                AbsoluteLayout.SetLayoutBounds(panelAttributo1, new Rectangle(0, 0, .5, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo1, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo2, new Rectangle(1, 0, .5, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo2, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo3, new Rectangle(0, 0, .5, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo3, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo4, new Rectangle(1, 0, .5, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo4, AbsoluteLayoutFlags.All);
                AbsoluteLayout.SetLayoutBounds(panelAttributo5, new Rectangle(0, 0, .5, 1));
                AbsoluteLayout.SetLayoutFlags(panelAttributo5, AbsoluteLayoutFlags.All);
            }
        }

        private void SetForm()
        {
            if (Modello.IsAttributo4Multiplo)
            {
                comboAttributi4Multi.DataSource = Modello.RisposteDettaglioAttributoList4;
                comboAttributi4Multi.SelectedItem = Modello.SelectedListAttributo4;
            }
            else
            {
                comboAttributi4.DataSource = Modello.RisposteDettaglioAttributoList4;
                comboAttributi4.SelectedItem = Modello.SelectedAttributo4;
            }

            comboStati.DataSource = Modello.ListaStati;
            comboStati.SelectedItem = Modello.Stato;

            comboAttributi1.SelectionChanged += ComboBox_SelectionChanged;
            comboAttributi2.SelectionChanged += ComboBox_SelectionChanged;
            comboAttributi3.SelectionChanged += ComboBox_SelectionChanged;
            comboAttributi4.SelectionChanged += ComboBox_SelectionChanged;
            comboAttributi4Multi.SelectionChanged += SfComboBox_SelectionChanged;
            comboAttributi5.SelectionChanged += ComboBox_SelectionChanged;

            content.IsVisible = true;
            busyindicator.IsVisible = false;
            busyindicator.IsEnabled = false;
        }

        private void SvuotaForm()
        {
            comboAttributi1.SelectedItem = Modello.SelectedAttributo1;
            comboAttributi2.SelectedItem = Modello.SelectedAttributo2;
            comboAttributi3.SelectedItem = Modello.SelectedAttributo3;
            comboAttributi4Multi.SelectedItem = Modello.SelectedListAttributo4;
            comboAttributi4.SelectedItem = Modello.SelectedAttributo4;
            comboAttributi5.SelectedItem = Modello.SelectedAttributo5;

            comboStati.SelectedItem = Modello.Stato;

            if (Modello.SelectedListAttributo4.Count == 0)
                comboAttributi4Multi.Clear();
        }

        private async Task TornaIndietro()
        {
            await Navigation.PopAsync();
        }

        //private async Task GoToEdit()
        //{
        //    Page page = Navigation.NavigationStack[Navigation.NavigationStack.Count - 1];
        //    Navigation.RemovePage(page);
        //    await Navigation.PopAsync();
        //}

        private void SfComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            List<object> items = new List<object>();

            if (e.Value != null && (((e.Value.GetType() == typeof(Attributo4) && (Attributo4)e.Value != null)) || (e.Value is IList && (e.Value as IList).Count > 0)))
            {
                if (Device.RuntimePlatform == Device.UWP)
                {
                    foreach (var item in e.Value as ObservableCollection<object>)
                    {
                        items.Add((Attributo4)item);
                    }
                }
                else
                {
                    for (int i = 0; i < (e.Value as List<object>).Count; i++)
                    {
                        object collection = (e.Value as List<object>)[i];
                        items.Add((Attributo4)collection);
                    }
                }

                Modello.SelectedListAttributo4 = items;
            }
            else
                Modello.SelectedListAttributo4 = new List<object>();

            if (Modello.Dettaglio != null && Modello.Dettaglio.Count() > 0)
                Modello.ListaAllegatiVisibile = Modello.CheckFormUpdate();
            else
                Modello.ListaAllegatiVisibile = Modello.CheckFormInsert();
        }

        private void ShowAttachmentMenu()
        {
            attachmentMenu.IsVisible = true;
        }

        private void HideAttachmentMenu()
        {
            attachmentMenu.IsVisible = false;
        }

        private void HandleAttach_Tapped(object sender, EventArgs e)
        {
            HideAttachmentMenu();
        }

        private void Button_Clicked(object sender, EventArgs e)
        {
            if (Modello.Dettaglio != null && Modello.Dettaglio.Count() > 0)
                Modello.UpdateDetailCommand.Execute("S");
            else
                Modello.SaveDetailCommand.Execute("S");
        }

        private void StackLayout_Tapped(object sender, EventArgs e)
        {
            if (Modello.Dettaglio != null && Modello.Dettaglio.Count() > 0)
                Modello.UpdateDetailCommand.Execute("S");
            else
                Modello.SaveDetailCommand.Execute("S");
        }

        private void Entry_Unfocused(object sender, FocusEventArgs e)
        {
            if (Modello.Dettaglio != null && Modello.Dettaglio.Count() > 0)
                Modello.ListaAllegatiVisibile = Modello.CheckFormUpdate() && Modello.AllegatiDisponibili.Count() > 0;
            else
                Modello.ListaAllegatiVisibile = Modello.CheckFormInsert() && Modello.AllegatiDisponibili.Count() > 0;
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if (e.Value as RisposteDettaglioAttributo != null)
                {
                    if ((e.Value as RisposteDettaglioAttributo).Ordine == 0)
                        (sender as SfComboBox).SelectedItem = null;
                }

                if (e.Value as Attributo4 != null)
                {
                    if ((e.Value as Attributo4).Ordine == 0)
                        (sender as SfComboBox).SelectedItem = null;

                    if (Modello.IsVisibleStato)
                    {
                        Modello.GestStati((e.Value as Attributo4).IdAttributo);

                        comboStati.DataSource = Modello.ListaStati;
                        comboStati.SelectedItem = Modello.Stato;
                    }
                }
            }
            catch (Exception)
            {

            }

            if (Modello.Dettaglio != null && Modello.Dettaglio.Count() > 0)
                Modello.ListaAllegatiVisibile = Modello.CheckFormUpdate() && Modello.AllegatiDisponibili.Count() > 0;
            else
                Modello.ListaAllegatiVisibile = Modello.CheckFormInsert() && Modello.AllegatiDisponibili.Count() > 0;
        }
        #endregion
    }
}