﻿using AuditManager.Data;
using AuditManager.Helpers;
using AuditManager.Interfaces;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Resx;
using AuditManager.Shared.Constants;
using AuditManager.Shared.Extensions;
using AuditManager.Shared.Types;
using AuditManager.Views;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Essentials;
using Xamarin.Forms;

namespace AuditManager.ViewModels
{
    public class DettaglioViewModel : BaseViewModel
    {
        #region Props
        private Audit audit;
        public Audit Audit
        {
            get { return audit; }
            set
            {
                audit = value;
                OnPropertyChanged(nameof(Audit));
            }
        }

        private AuditChecklist checklist;
        public AuditChecklist Checklist
        {
            get { return checklist; }
            set
            {
                checklist = value;
                OnPropertyChanged(nameof(Checklist));
            }
        }

        private AuditCategorie categoria;
        public AuditCategorie Categoria
        {
            get { return categoria; }
            set
            {
                categoria = value;
                OnPropertyChanged(nameof(Categoria));
            }
        }

        private AuditDomandeRisposte domanda;
        public AuditDomandeRisposte Domanda
        {
            get { return domanda; }
            set
            {
                domanda = value;
                OnPropertyChanged(nameof(Domanda));
            }
        }

        private AuditRisposteDettaglio[] dettaglio;
        public AuditRisposteDettaglio[] Dettaglio
        {
            get { return dettaglio; }
            set
            {
                dettaglio = value;
                OnPropertyChanged(nameof(Dettaglio));
            }
        }

        private AuditRisposteDettaglio dettaglioAperto;
        public AuditRisposteDettaglio DettaglioAperto
        {
            get { return dettaglioAperto; }
            set
            {
                dettaglioAperto = value;
                OnPropertyChanged(nameof(DettaglioAperto));
            }
        }

        private AuditRisposteDettaglio oldDettaglioAperto;
        public AuditRisposteDettaglio OldDettaglioAperto
        {
            get { return oldDettaglioAperto; }
            set
            {
                oldDettaglioAperto = value;
                OnPropertyChanged(nameof(OldDettaglioAperto));
            }
        }

        private string pageTitle = string.Empty;
        public string PageTitle
        {
            get { return pageTitle; }
            set
            {
                pageTitle = value;
                OnPropertyChanged(nameof(PageTitle));
            }
        }

        private AuditTipiAudit TipoAudit { get; set; }
        private AlertModel AlertModel { get; set; }

        private AuditRisposteDettaglio[] dettagliInseriti;
        public AuditRisposteDettaglio[] DettagliInseriti
        {
            get { return dettagliInseriti; }
            set
            {
                dettagliInseriti = value;
                OnPropertyChanged(nameof(DettagliInseriti));
            }
        }

        private string messaggioInserimentoOk = string.Empty;
        public string MessaggioInserimentoOk
        {
            get { return messaggioInserimentoOk; }
            set
            {
                messaggioInserimentoOk = value;
                OnPropertyChanged(nameof(MessaggioInserimentoOk));
            }
        }

        private ObservableCollection<AuditAllegatiRisposta> listaAllegati;
        public ObservableCollection<AuditAllegatiRisposta> ListaAllegati
        {
            get { return listaAllegati; }
            set
            {
                listaAllegati = value;
                OnPropertyChanged(nameof(ListaAllegati));

            }
        }

        private bool listaAllegatiVisibile = false;
        public bool ListaAllegatiVisibile
        {
            get { return listaAllegatiVisibile; }
            set
            {
                listaAllegatiVisibile = value;
                OnPropertyChanged(nameof(ListaAllegatiVisibile));
            }
        }

        private ObservableCollection<BottoneDomanda> allegatiDisponibili;
        public ObservableCollection<BottoneDomanda> AllegatiDisponibili
        {
            get { return allegatiDisponibili; }
            set
            {
                allegatiDisponibili = value;
                OnPropertyChanged(nameof(AllegatiDisponibili));
            }
        }
        #endregion

        #region Text Props
        private bool isTextEnabled = false;
        public bool IsTextEnabled
        {
            get { return isTextEnabled; }
            set
            {
                isTextEnabled = value;
                OnPropertyChanged(nameof(IsTextEnabled));
            }
        }

        private string testoNc1 = string.Empty;
        public string TestoNc1
        {
            get { return testoNc1; }
            set
            {
                if (testoNc1 != value)
                    IsDirty = true;

                testoNc1 = value;
                OnPropertyChanged(nameof(TestoNc1));
            }
        }

        private string testoNc2 = string.Empty;
        public string TestoNc2
        {
            get { return testoNc2; }
            set
            {
                if (testoNc2 != value)
                    IsDirty = true;

                testoNc2 = value;
                OnPropertyChanged(nameof(TestoNc2));
            }
        }

        private string testoNc3 = string.Empty;
        public string TestoNc3
        {
            get { return testoNc3; }
            set
            {
                if (testoNc3 != value)
                    IsDirty = true;

                testoNc3 = value;
                OnPropertyChanged(nameof(TestoNc3));
            }
        }

        private string testoNc4 = string.Empty;
        public string TestoNc4
        {
            get { return testoNc4; }
            set
            {
                if (testoNc4 != value)
                    IsDirty = true;

                testoNc4 = value;
                OnPropertyChanged(nameof(TestoNc4));
            }
        }

        private string placeholderTestoNc1 = string.Empty;
        public string PlaceholderTestoNc1
        {
            get { return placeholderTestoNc1; }
            set
            {
                placeholderTestoNc1 = value;
                OnPropertyChanged(nameof(PlaceholderTestoNc1));
            }
        }

        private string placeholderTestoNc2 = string.Empty;
        public string PlaceholderTestoNc2
        {
            get { return placeholderTestoNc2; }
            set
            {
                placeholderTestoNc2 = value;
                OnPropertyChanged(nameof(PlaceholderTestoNc2));
            }
        }

        private string placeholderTestoNc3 = string.Empty;
        public string PlaceholderTestoNc3
        {
            get { return placeholderTestoNc3; }
            set
            {
                placeholderTestoNc3 = value;
                OnPropertyChanged(nameof(PlaceholderTestoNc3));
            }
        }

        private string placeholderTestoNc4 = string.Empty;
        public string PlaceholderTestoNc4
        {
            get { return placeholderTestoNc4; }
            set
            {
                placeholderTestoNc4 = value;
                OnPropertyChanged(nameof(PlaceholderTestoNc4));
            }
        }

        private bool isVisibleTestoNc2 = false;
        public bool IsVisibleTestoNc2
        {
            get { return isVisibleTestoNc2; }
            set
            {
                isVisibleTestoNc2 = value;
                OnPropertyChanged(nameof(IsVisibleTestoNc2));
            }
        }

        private bool isVisibleTestoNc3 = false;
        public bool IsVisibleTestoNc3
        {
            get { return isVisibleTestoNc3; }
            set
            {
                isVisibleTestoNc3 = value;
                OnPropertyChanged(nameof(IsVisibleTestoNc3));
            }
        }

        private bool isVisibleTestoNc4 = false;
        public bool IsVisibleTestoNc4
        {
            get { return isVisibleTestoNc4; }
            set
            {
                isVisibleTestoNc4 = value;
                OnPropertyChanged(nameof(IsVisibleTestoNc4));
            }
        }

        private string nota = string.Empty;
        public string Nota
        {
            get { return nota; }
            set
            {
                if (nota != value)
                    IsDirty = true;

                nota = value;
                OnPropertyChanged(nameof(Nota));
            }
        }
        #endregion

        #region Combobox Props
        private bool isVisibleAttributo1 = false;
        public bool IsVisibleAttributo1
        {
            get { return isVisibleAttributo1; }
            set
            {
                isVisibleAttributo1 = value;
                OnPropertyChanged(nameof(IsVisibleAttributo1));
            }
        }

        private bool isVisibleAttributo2 = false;
        public bool IsVisibleAttributo2
        {
            get { return isVisibleAttributo2; }
            set
            {
                isVisibleAttributo2 = value;
                OnPropertyChanged(nameof(IsVisibleAttributo2));
            }
        }

        private bool isVisibleAttributo3 = false;
        public bool IsVisibleAttributo3
        {
            get { return isVisibleAttributo3; }
            set
            {
                isVisibleAttributo3 = value;
                OnPropertyChanged(nameof(IsVisibleAttributo3));
            }
        }

        private bool isVisibleAttributo4 = false;
        public bool IsVisibleAttributo4
        {
            get { return isVisibleAttributo4; }
            set
            {
                isVisibleAttributo4 = value;
                OnPropertyChanged(nameof(IsVisibleAttributo4));
            }
        }

        private bool isAttributo4Multiplo = false;
        public bool IsAttributo4Multiplo
        {
            get { return isAttributo4Multiplo; }
            set
            {
                isAttributo4Multiplo = value;
                OnPropertyChanged(nameof(IsAttributo4Multiplo));
            }
        }

        private bool isVisibleAttributo5 = false;
        public bool IsVisibleAttributo5
        {
            get { return isVisibleAttributo5; }
            set
            {
                isVisibleAttributo5 = value;
                OnPropertyChanged(nameof(IsVisibleAttributo5));
            }
        }

        private bool isVisibleStato = false;
        public bool IsVisibleStato
        {
            get { return isVisibleStato; }
            set
            {
                isVisibleStato = value;
                OnPropertyChanged(nameof(IsVisibleStato));
            }
        }

        private string placeholderAttributo1 = string.Empty;
        public string PlaceholderAttributo1
        {
            get { return placeholderAttributo1; }
            set
            {
                placeholderAttributo1 = value;
                OnPropertyChanged(nameof(PlaceholderAttributo1));
            }
        }

        private string placeholderAttributo2 = string.Empty;
        public string PlaceholderAttributo2
        {
            get { return placeholderAttributo2; }
            set
            {
                placeholderAttributo2 = value;
                OnPropertyChanged(nameof(PlaceholderAttributo2));
            }
        }

        private string placeholderAttributo3 = string.Empty;
        public string PlaceholderAttributo3
        {
            get { return placeholderAttributo3; }
            set
            {
                placeholderAttributo3 = value;
                OnPropertyChanged(nameof(PlaceholderAttributo3));
            }
        }

        private string placeholderAttributo4 = string.Empty;
        public string PlaceholderAttributo4
        {
            get { return placeholderAttributo4; }
            set
            {
                placeholderAttributo4 = value;
                OnPropertyChanged(nameof(PlaceholderAttributo4));
            }
        }

        private string placeholderAttributo5 = string.Empty;
        public string PlaceholderAttributo5
        {
            get { return placeholderAttributo5; }
            set
            {
                placeholderAttributo5 = value;
                OnPropertyChanged(nameof(PlaceholderAttributo5));
            }
        }

        private List<RisposteDettaglioAttributo> risposteDettaglioAttributoList1;
        public List<RisposteDettaglioAttributo> RisposteDettaglioAttributoList1
        {
            get { return risposteDettaglioAttributoList1; }
            set
            {
                risposteDettaglioAttributoList1 = value;
                OnPropertyChanged(nameof(RisposteDettaglioAttributoList1));
            }
        }

        private List<RisposteDettaglioAttributo> risposteDettaglioAttributoList2;
        public List<RisposteDettaglioAttributo> RisposteDettaglioAttributoList2
        {
            get { return risposteDettaglioAttributoList2; }
            set
            {
                risposteDettaglioAttributoList2 = value;
                OnPropertyChanged(nameof(RisposteDettaglioAttributoList2));
            }
        }

        private List<RisposteDettaglioAttributo> risposteDettaglioAttributoList3;
        public List<RisposteDettaglioAttributo> RisposteDettaglioAttributoList3
        {
            get { return risposteDettaglioAttributoList3; }
            set
            {
                risposteDettaglioAttributoList3 = value;
                OnPropertyChanged(nameof(RisposteDettaglioAttributoList3));
            }
        }

        private List<object> risposteDettaglioAttributoList4;
        public List<object> RisposteDettaglioAttributoList4
        {
            get { return risposteDettaglioAttributoList4; }
            set
            {
                risposteDettaglioAttributoList4 = value;
                OnPropertyChanged(nameof(RisposteDettaglioAttributoList4));
            }
        }

        private List<RisposteDettaglioAttributo> risposteDettaglioAttributoList5;
        public List<RisposteDettaglioAttributo> RisposteDettaglioAttributoList5
        {
            get { return risposteDettaglioAttributoList5; }
            set
            {
                risposteDettaglioAttributoList5 = value;
                OnPropertyChanged(nameof(RisposteDettaglioAttributoList5));
            }
        }

        private RisposteDettaglioAttributo selectedAttributo1;
        public RisposteDettaglioAttributo SelectedAttributo1
        {
            get { return selectedAttributo1; }
            set
            {
                if (selectedAttributo1 != value)
                    IsDirty = true;

                selectedAttributo1 = value;
                OnPropertyChanged(nameof(SelectedAttributo1));
            }
        }

        private RisposteDettaglioAttributo selectedAttributo2;
        public RisposteDettaglioAttributo SelectedAttributo2
        {
            get { return selectedAttributo2; }
            set
            {
                if (selectedAttributo2 != value)
                    IsDirty = true;

                selectedAttributo2 = value;
                OnPropertyChanged(nameof(SelectedAttributo2));
            }
        }

        private RisposteDettaglioAttributo selectedAttributo3;
        public RisposteDettaglioAttributo SelectedAttributo3
        {
            get { return selectedAttributo3; }
            set
            {
                if (selectedAttributo3 != value)
                    IsDirty = true;

                selectedAttributo3 = value;
                OnPropertyChanged(nameof(SelectedAttributo3));
            }
        }

        private List<object> selectedListAttributo4;
        public List<object> SelectedListAttributo4
        {
            get { return selectedListAttributo4; }
            set
            {
                if (selectedListAttributo4 != value)
                    IsDirty = true;

                selectedListAttributo4 = value;
                OnPropertyChanged(nameof(SelectedListAttributo4));
            }
        }

        private Attributo4 selectedAttributo4;
        public Attributo4 SelectedAttributo4
        {
            get { return selectedAttributo4; }
            set
            {
                if (selectedAttributo4 != value)
                    IsDirty = true;

                selectedAttributo4 = value;
                OnPropertyChanged(nameof(SelectedAttributo4));
            }
        }

        private RisposteDettaglioAttributo selectedAttributo5;
        public RisposteDettaglioAttributo SelectedAttributo5
        {
            get { return selectedAttributo5; }
            set
            {
                if (selectedAttributo5 != value)
                    IsDirty = true;

                selectedAttributo5 = value;
                OnPropertyChanged(nameof(SelectedAttributo5));
            }
        }

        private ComboboxItem stato;
        public ComboboxItem Stato
        {
            get { return stato; }
            set
            {
                if (stato != value)
                    IsDirty = true;

                stato = value;
                OnPropertyChanged(nameof(Stato));

                if (Stato != null && !string.IsNullOrEmpty(Stato.Colore))
                    ColoreStato = FunctionsHelper.GetColorFromHexString(Stato.Colore, false);
            }
        }

        private Color coloreStato;
        public Color ColoreStato
        {
            get { return coloreStato; }
            set
            {
                coloreStato = value;
                OnPropertyChanged(nameof(ColoreStato));
            }
        }

        private List<ComboboxItem> listaStati;
        public List<ComboboxItem> ListaStati
        {
            get { return listaStati; }
            set
            {
                listaStati = value;
                OnPropertyChanged(nameof(ListaStati));
            }
        }
        #endregion

        #region Props Risoluzione
        private string dataPrevistaRisoluzioneAsString;
        public string DataPrevistaRisoluzioneAsString
        {
            get { return dataPrevistaRisoluzioneAsString; }
            set
            {
                dataPrevistaRisoluzioneAsString = value;
                OnPropertyChanged(nameof(DataPrevistaRisoluzioneAsString));
            }
        }

        private ObservableCollection<object> dataPrevistaRisoluzioneAsDate;
        public ObservableCollection<object> DataPrevistaRisoluzioneAsDate
        {
            get { return dataPrevistaRisoluzioneAsDate; }
            set
            {
                if (dataPrevistaRisoluzioneAsDate != value)
                    IsDirty = true;

                dataPrevistaRisoluzioneAsDate = value;
                OnPropertyChanged(nameof(DataPrevistaRisoluzioneAsDate));
            }
        }

        private ObservableCollection<object> Oggi;
        #endregion

        #region Commands
        public Command<string> SaveDetailCommand { get; set; }
        public Command<string> UpdateDetailCommand { get; set; }
        public Command CancellaDettaglioCommand { get; set; }
        public Command ResetDetailCommand { get; set; }
        public Command OpenPickerCommand { get; set; }
        public Command<ObservableCollection<object>> DataPrevistaRisoluzioneCommand { get; set; }

        public Command AddAttachFsCommand { get; set; }
        public Command AddAttachPhotoCommand { get; set; }
        public Command AddGalleryCommand { get; set; }
        public Command AddAttachDrawingCommand { get; set; }
        public Command AddAttachVideoCommand { get; set; }
        public Command AddAttachAudioCommand { get; set; }
        public Command AddAttachEvidenzaCommand { get; set; }
        public Command AddAttachTabellaCommand { get; set; }
        public Command AddAttachDocumentiCommand { get; set; }
        public Command AddAttachMappaCommand { get; set; }
        public Command AddAttachBarcodeCommand { get; set; }

        public Command<string> OpenAttachCommand { get; set; }
        public Command<int> DeleteAttachCommand { get; set; }
        #endregion

        #region Ctors
        public DettaglioViewModel()
        {

        }

        public DettaglioViewModel(NavigationParameterModel parametro) : base()
        {
            IsVisible = true;
            IsOnline = Settings.IsAppOnline;

            Audit = parametro.Audit;
            Checklist = parametro.Checklist;
            Categoria = parametro.Categoria;
            Domanda = parametro.Domanda;
            Dettaglio = parametro.Dettaglio;
            PageTitle = $"{Categoria.DescCategoria} - {Domanda.Testo}";

            if (Dettaglio != null && Dettaglio.Count() > 0)
            {
                DettaglioAperto = Dettaglio?.FirstOrDefault();
                OldDettaglioAperto = DettaglioAperto.CloneJson();
                IsVisibleStato = true;
                ListaAllegatiVisibile = true;
            }
            else
            {
                DettaglioAperto = new AuditRisposteDettaglio();
                ListaAllegatiVisibile = false;
            }

            TipoAudit = App.Database.GetAuditTipiAudit(Audit.IdAudit).FirstOrDefault();

            AlertModel = new AlertModel
            {
                Title = AppResources.StringMissingFieldsFormTitle,
                OkButton = AppResources.LabelOK
            };

            ListaAllegati = new ObservableCollection<AuditAllegatiRisposta>();
            AllegatiDisponibili = new ObservableCollection<BottoneDomanda>();

            SaveDetailCommand = new Command<string>(async (s) => await ExecuteSaveDetailCommand(s));
            UpdateDetailCommand = new Command<string>(async (s) => await ExecuteUpdateDetailCommand(s));
            CancellaDettaglioCommand = new Command(async () => await ExecuteCancellaDettaglioCommand());
            ResetDetailCommand = new Command(() => ExecuteResetDetailCommand());
            OpenPickerCommand = new Command(() => ExecuteOpenPickerCommand());
            DataPrevistaRisoluzioneCommand = new Command<ObservableCollection<object>>((obj) => ExecuteDataPrevistaRisoluzioneCommand(obj));
            AddAttachFsCommand = new Command(async () => await ExecuteAddAttachFsCommand());
            AddAttachPhotoCommand = new Command(async () => await ExecuteAddAttachPhotoCommand());
            AddGalleryCommand = new Command(async () => await ExecuteAddGalleryCommand());
            AddAttachDrawingCommand = new Command(async () => await ExecuteAddAttachDrawingCommand());
            AddAttachVideoCommand = new Command(async () => await ExecuteAddAttachVideoCommand());
            AddAttachAudioCommand = new Command(async () => await ExecuteAddAttachAudioCommand());
            AddAttachEvidenzaCommand = new Command(async () => await ExecuteAddAttachEvidenzaCommand());
            AddAttachTabellaCommand = new Command(async () => await ExecuteAddAttachTabellaCommand());
            AddAttachDocumentiCommand = new Command(async () => await ExecuteAddAttachDocumentiCommand());
            AddAttachMappaCommand = new Command(async () => await ExecuteAddAttachMappaCommand());
            AddAttachBarcodeCommand = new Command(async () => await ExecuteAddAttachBarcodeCommand());
            SyncCommand = new Command(async () => await ExecuteSyncCommand());

            OpenAttachCommand = new Command<string>(async (id) => await ExecuteOpenAttachCommand(id));
            DeleteAttachCommand = new Command<int>(async (id) => await ExecuteDeleteAttachCommand(id));
        }
        #endregion

        #region Common Methods
        public void InitText()
        {
            IsVisibleTestoNc2 = false;
            IsVisibleTestoNc3 = false;
            IsVisibleTestoNc4 = false;

            string[] placeholderTextArray = TipoAudit.PlaceholderTestoNc?.Split(new[] { "|" }, StringSplitOptions.RemoveEmptyEntries);
            string[] testoArray = new string[4];

            if (!string.IsNullOrEmpty(DettaglioAperto.Testo))
            {
                string[] tmpArray = DettaglioAperto.Testo.Split(new[] { "|" }, StringSplitOptions.RemoveEmptyEntries);

                for (int i = 0; i < tmpArray.Length; i++)
                    testoArray[i] = tmpArray[i];
            }

            switch (TipoAudit.NrCampiTestoNc)
            {
                case 1:
                    PlaceholderTestoNc1 = placeholderTextArray[0];
                    TestoNc1 = testoArray[0];
                    break;
                case 2:
                    IsVisibleTestoNc2 = true;

                    PlaceholderTestoNc1 = placeholderTextArray[0];
                    PlaceholderTestoNc2 = placeholderTextArray[1];
                    TestoNc1 = testoArray[0];
                    TestoNc2 = testoArray[1];
                    break;
                case 3:
                    IsVisibleTestoNc2 = true;
                    IsVisibleTestoNc3 = true;

                    PlaceholderTestoNc1 = placeholderTextArray[0];
                    PlaceholderTestoNc2 = placeholderTextArray[1];
                    PlaceholderTestoNc3 = placeholderTextArray[2];
                    TestoNc1 = testoArray[0];
                    TestoNc2 = testoArray[1];
                    TestoNc3 = testoArray[2];
                    break;
                case 4:
                    IsVisibleTestoNc2 = true;
                    IsVisibleTestoNc3 = true;
                    IsVisibleTestoNc4 = true;

                    PlaceholderTestoNc1 = placeholderTextArray[0];
                    PlaceholderTestoNc2 = placeholderTextArray[1];
                    PlaceholderTestoNc3 = placeholderTextArray[2];
                    PlaceholderTestoNc4 = placeholderTextArray[3];
                    TestoNc1 = testoArray[0];
                    TestoNc2 = testoArray[1];
                    TestoNc3 = testoArray[2];
                    TestoNc4 = testoArray[3];
                    break;
                default:
                    break;
            }

            PlaceholderTestoNc1 = $"{PlaceholderTestoNc1} *";
        }

        public void InitCombobox()
        {
            RisposteDettaglioMacroAttributo risposteDettaglioMacroAttributo1 = null;
            RisposteDettaglioMacroAttributo risposteDettaglioMacroAttributo2 = null;
            RisposteDettaglioMacroAttributo risposteDettaglioMacroAttributo3 = null;
            RisposteDettaglioMacroAttributo risposteDettaglioMacroAttributo4 = null;
            RisposteDettaglioMacroAttributo risposteDettaglioMacroAttributo5 = null;

            RisposteDettaglioAttributoList1 = new List<RisposteDettaglioAttributo>();
            RisposteDettaglioAttributoList2 = new List<RisposteDettaglioAttributo>();
            RisposteDettaglioAttributoList3 = new List<RisposteDettaglioAttributo>();
            RisposteDettaglioAttributoList4 = new List<object>();
            RisposteDettaglioAttributoList5 = new List<RisposteDettaglioAttributo>();
            SelectedListAttributo4 = new List<object>();
            ListaStati = new List<ComboboxItem>();

            IsVisibleAttributo1 = (TipoAudit.NrAttributiNC > 0) && AttributiVisibile;
            IsVisibleAttributo2 = (TipoAudit.NrAttributiNC > 1) && AttributiVisibile;
            IsVisibleAttributo3 = (TipoAudit.NrAttributiNC > 2) && AttributiVisibile;
            IsVisibleAttributo4 = (TipoAudit.NrAttributiNC > 3) && AttributiVisibile;
            IsVisibleAttributo5 = (TipoAudit.NrAttributiNC > 4) && AttributiVisibile;

            IsAttributo4Multiplo = TipoAudit.TipoInCaricoA?.Equals("M") == true && !IsVisibleStato;

            if (TipoAudit.NrAttributiNC > 0)
            {
                risposteDettaglioMacroAttributo1 = App.Database.GetRisposteDettaglioMacroAttributo(1, TipoAudit.IdTipoAudit)?.FirstOrDefault();
                PlaceholderAttributo1 = risposteDettaglioMacroAttributo1?.DescRisposteDettaglioMacroAttributo;
                DettaglioAperto.IdRisposteDettaglioMacroAttributo1 = risposteDettaglioMacroAttributo1?.IdRisposteDettaglioMacroAttributo;
                RisposteDettaglioAttributoList1 = App.Database.GetRisposteDettaglioAttributo((int)DettaglioAperto.IdRisposteDettaglioMacroAttributo1, Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo1);

                if (TipoAudit.NrAttributiNC > 1)
                {
                    risposteDettaglioMacroAttributo2 = App.Database.GetRisposteDettaglioMacroAttributo(2, TipoAudit.IdTipoAudit)?.FirstOrDefault();
                    PlaceholderAttributo2 = risposteDettaglioMacroAttributo2?.DescRisposteDettaglioMacroAttributo;
                    DettaglioAperto.IdRisposteDettaglioMacroAttributo2 = risposteDettaglioMacroAttributo2?.IdRisposteDettaglioMacroAttributo;
                    RisposteDettaglioAttributoList2 = App.Database.GetRisposteDettaglioAttributo((int)DettaglioAperto.IdRisposteDettaglioMacroAttributo2, Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo2);

                    if (TipoAudit.NrAttributiNC > 2)
                    {
                        risposteDettaglioMacroAttributo3 = App.Database.GetRisposteDettaglioMacroAttributo(3, TipoAudit.IdTipoAudit)?.FirstOrDefault();
                        PlaceholderAttributo3 = risposteDettaglioMacroAttributo3?.DescRisposteDettaglioMacroAttributo;
                        DettaglioAperto.IdRisposteDettaglioMacroAttributo3 = risposteDettaglioMacroAttributo3?.IdRisposteDettaglioMacroAttributo;
                        RisposteDettaglioAttributoList3 = App.Database.GetRisposteDettaglioAttributo((int)DettaglioAperto.IdRisposteDettaglioMacroAttributo3, Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo3);

                        if (TipoAudit.NrAttributiNC > 3)
                        {
                            risposteDettaglioMacroAttributo4 = App.Database.GetRisposteDettaglioMacroAttributo(4, TipoAudit.IdTipoAudit)?.FirstOrDefault();
                            PlaceholderAttributo4 = risposteDettaglioMacroAttributo4?.DescRisposteDettaglioMacroAttributo;
                            DettaglioAperto.IdRisposteDettaglioMacroAttributo4 = risposteDettaglioMacroAttributo4?.IdRisposteDettaglioMacroAttributo;

                            ParametriGlobali parametro4 = App.Database.GetParametroGlobale(Constants.ConstUsaAttributo4);

                            if (parametro4.ValoreParametroBool == true)
                            {
                                List<RisposteDettaglioAttributo> lista = App.Database.GetRisposteDettaglioAttributo((int)DettaglioAperto.IdRisposteDettaglioMacroAttributo4, Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo4, DettaglioAperto?.NcMultipla);
                                foreach (RisposteDettaglioAttributo item in lista)
                                {
                                    RisposteDettaglioAttributoList4.Add(new Attributo4() { IdAttributo = item.IdRisposteDettaglioAttributo, DescAttributo = item.DescRisposteDettaglioAttributo, Ordine = item.Ordine, IsPlaceholder = item.IsPlaceholder });
                                }
                            }
                            else
                            {
                                List<Auditor> lista = App.Database.GetAuditors(Audit.IdSoggetto, DettaglioAperto?.NcMultipla);
                                foreach (Auditor item in lista)
                                {
                                    if (item.IsPlaceholder)
                                        RisposteDettaglioAttributoList4.Add(new Attributo4() { IdAttributo = item.IdAuditor, DescAttributo = risposteDettaglioMacroAttributo4?.DescRisposteDettaglioMacroAttributo, Ordine = item.Ordine, IsPlaceholder = item.IsPlaceholder });
                                    else
                                        RisposteDettaglioAttributoList4.Add(new Attributo4() { IdAttributo = item.IdAuditor, DescAttributo = $"{item.NomeAuditor} {item.CognomeAuditor}".Trim(), Ordine = item.Ordine, IsPlaceholder = item.IsPlaceholder });
                                }
                            }

                            if (TipoAudit.NrAttributiNC > 4)
                            {
                                risposteDettaglioMacroAttributo5 = App.Database.GetRisposteDettaglioMacroAttributo(5, TipoAudit.IdTipoAudit)?.FirstOrDefault();
                                PlaceholderAttributo5 = risposteDettaglioMacroAttributo5?.DescRisposteDettaglioMacroAttributo;
                                DettaglioAperto.IdRisposteDettaglioMacroAttributo5 = risposteDettaglioMacroAttributo5?.IdRisposteDettaglioMacroAttributo;
                                RisposteDettaglioAttributoList5 = App.Database.GetRisposteDettaglioAttributo((int)DettaglioAperto.IdRisposteDettaglioMacroAttributo5, Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo5);
                            }
                        }
                    }
                }
            }

            if (TipoAudit.NrAttributiObb >= 1)
                PlaceholderAttributo1 = $"{PlaceholderAttributo1} *";

            if (TipoAudit.NrAttributiObb >= 2)
                PlaceholderAttributo2 = $"{PlaceholderAttributo2} *";

            if (TipoAudit.NrAttributiObb >= 3)
                PlaceholderAttributo3 = $"{PlaceholderAttributo3} *";

            if (TipoAudit.NrAttributiObb >= 4)
                PlaceholderAttributo4 = $"{PlaceholderAttributo4} *";

            if (TipoAudit.NrAttributiObb >= 5)
                PlaceholderAttributo5 = $"{PlaceholderAttributo5} *";

            if (Dettaglio != null && Dettaglio.Count() > 0)
            {
                SelectedAttributo1 = RisposteDettaglioAttributoList1.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == DettaglioAperto.IdRisposteDettaglioAttributo1);
                SelectedAttributo2 = RisposteDettaglioAttributoList2.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == DettaglioAperto.IdRisposteDettaglioAttributo2);
                SelectedAttributo3 = RisposteDettaglioAttributoList3.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == DettaglioAperto.IdRisposteDettaglioAttributo3);

                foreach (object attr4 in RisposteDettaglioAttributoList4)
                {
                    if (((Attributo4)attr4).IdAttributo == DettaglioAperto.IdRisposteDettaglioAttributo4)
                        SelectedAttributo4 = (Attributo4)attr4;
                }

                SelectedAttributo5 = RisposteDettaglioAttributoList5.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == DettaglioAperto.IdRisposteDettaglioAttributo5);

                List<StatiDomandeRisposteDettaglio> listaStatiFiltrata = App.Database.GetStatiNCFilteredByInCarico(Audit.IdAudit, DettaglioAperto?.IdRisposteDettaglioAttributo4, DettaglioAperto.IdStatoNC);

                foreach (StatiDomandeRisposteDettaglio item in listaStatiFiltrata)
                {
                    ListaStati.Add(new ComboboxItem()
                    {
                        ID = item.IdStatoNC,
                        Name = (App.CultureInfo.TwoLetterISOLanguageName == "it") ? item.DescStatoNC : item.DescStatoEng,
                        Colore = item.ColoreStatoNC
                    });
                }

                Stato = ListaStati.FirstOrDefault(w => w.ID == DettaglioAperto.IdStatoNC);
            }
            else
            {
                SelectedAttributo1 = RisposteDettaglioAttributoList1.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == Domanda.IdRisposteDettaglioAttributo1);
                SelectedAttributo2 = RisposteDettaglioAttributoList2.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == Domanda.IdRisposteDettaglioAttributo2);
                SelectedAttributo3 = RisposteDettaglioAttributoList3.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == Domanda.IdRisposteDettaglioAttributo3);

                foreach (object attr4 in RisposteDettaglioAttributoList4)
                {
                    if (((Attributo4)attr4).IdAttributo == Domanda.IdRisposteDettaglioAttributo4)
                    {
                        SelectedAttributo4 = (Attributo4)attr4;
                        SelectedListAttributo4.Add((Attributo4)attr4);
                        break;
                    }
                    else
                    {
                        SelectedAttributo4 = null;
                        SelectedListAttributo4.Clear();
                    }
                }

                SelectedAttributo5 = RisposteDettaglioAttributoList5.FirstOrDefault(w => w.IdRisposteDettaglioAttributo == Domanda.IdRisposteDettaglioAttributo5);

                if (Stato == null)
                    Stato = new ComboboxItem() { ID = 1, Colore = "#000000" };
            }
        }

        public void InitRisoluzione()
        {
            Oggi = new ObservableCollection<object>();
            dataPrevistaRisoluzioneAsDate = new ObservableCollection<object>();

            if (DateTime.Now.Date.Day < 10)
                Oggi.Add("0" + DateTime.Now.Date.Day);
            else
                Oggi.Add(DateTime.Now.Date.Day.ToString());

            Oggi.Add(CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(DateTime.Now.Date.Month).Substring(0, 3));
            Oggi.Add(DateTime.Now.Date.Year.ToString());

            if (DettaglioAperto != null && DettaglioAperto?.DataPrevistaRisoluzione != null)
            {
                DateTime data = (DateTime)DettaglioAperto.DataPrevistaRisoluzione;

                if (data.Day < 10)
                    DataPrevistaRisoluzioneAsDate.Add("0" + data.Day);
                else
                    DataPrevistaRisoluzioneAsDate.Add(data.Day.ToString());

                DataPrevistaRisoluzioneAsDate.Add(CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(data.Month).Substring(0, 3));
                DataPrevistaRisoluzioneAsDate.Add(data.Year.ToString());

                DataPrevistaRisoluzioneAsString = AppResources.StringEntroData.Replace(Constants.DateTimeFormat, DateTime.Parse($"{DataPrevistaRisoluzioneAsDate[0].ToString()} {DataPrevistaRisoluzioneAsDate[1].ToString()} {DataPrevistaRisoluzioneAsDate[2].ToString()}").ToString(Constants.DateTimeFormat));
            }
            else
            {
                DataPrevistaRisoluzioneAsDate = Oggi;
                DataPrevistaRisoluzioneAsString = string.Empty;
            }

            Nota = DettaglioAperto.Nota;
        }

        public void GetAllegati()
        {
            List<AuditAllegatiRisposta> lista = new List<AuditAllegatiRisposta>();

            if (Dettaglio != null && Dettaglio.Count() > 0)
            {
                lista = App.Database.GetListAllegatiRisposta(Audit.IdAudit, Domanda.IdRispostaGuid, DettaglioAperto.IdRispostaDettaglioGuid);

                foreach (AuditAllegatiRisposta item in lista)
                    GetImmagineAllegato(item);
            }

            if (DettaglioAperto.IsMappata == true)
            {
                lista.Insert(0, new AuditAllegatiRisposta()
                {
                    PKID = -1,
                    NomeAllegato = AppResources.StringEditMappatura,
                    Immagine = FunctionsHelper.SetImageString("icon_azioni_mappatura_dark.png")
                });
            }

            ListaAllegati = new ObservableCollection<AuditAllegatiRisposta>(lista);
        }

        public void SetVisibilities()
        {
            IsEnabled = Audit.IdStato < 4 && !Categoria.Readonly;

            if (Dettaglio != null && Dettaglio.Count() > 0)
                IsTextEnabled = IsEnabled && DettaglioAperto?.IdStatoNC == 1;
            else
                IsTextEnabled = IsEnabled;

            SetVisibilities(Audit.IdAudit);
            
            AllegatoPrincipaleVisible = false;
            AddAuditVisible = false;
            SearchVisible = false;
            FilterVisible = false;
            FilterHomeVisible = false;
            ListaLavoriVisible = false;
            ListaLavoriCompletaVisible = false;
            DettaglioAuditVisible = false;
            NotificaListaLavoriVisible = false;
            CompletaVisible = false;

            AggiungiAllegatoMappaVisible = AggiungiAllegatoMappaVisible && DettaglioAperto.IsMappata != true;
        }

        public bool CheckFormInsert()
        {
            int NrAttrObbligatori = TipoAudit.NrAttributiObb;

            bool flgContinue = true;

            if (string.IsNullOrEmpty(TestoNc1))
                flgContinue = false;

            if (!flgContinue)
                AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionTesto;
            else
            {
                switch (NrAttrObbligatori)
                {
                    case 0:
                        flgContinue = true;
                        break;
                    case 1:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 2:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 3:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 4:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo4 != null && SelectedAttributo4.IdAttributo != 0 || SelectedListAttributo4.Count > 0);
                        break;
                    case 5:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo4 != null && SelectedAttributo4.IdAttributo != 0 || SelectedListAttributo4.Count > 0)
                            && (SelectedAttributo5 != null && SelectedAttributo5.IdRisposteDettaglioAttributo != 0);
                        break;
                    default:
                        break;
                }

                if (NrAttrObbligatori > 0)
                    AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionAttributiN.Replace("NNN", NrAttrObbligatori.ToString());
                else
                    AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionAttributi;
            }

            return flgContinue;
        }

        public bool CheckFormUpdate()
        {
            int NrAttrObbligatori = TipoAudit.NrAttributiObb;
            bool flgContinue = true;

            if (string.IsNullOrEmpty(TestoNc1))
                flgContinue = false;

            if (string.IsNullOrEmpty(TestoNc1))
                flgContinue = false;

            if (!flgContinue)
                AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionTesto;
            else
            {
                switch (NrAttrObbligatori)
                {
                    case 0:
                        flgContinue = true;
                        break;
                    case 1:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 2:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 3:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0);
                        break;
                    case 4:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo4 != null && SelectedAttributo4.IdAttributo != 0);
                        break;
                    case 5:
                        flgContinue = (SelectedAttributo1 != null && SelectedAttributo1.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo2 != null && SelectedAttributo2.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo3 != null && SelectedAttributo3.IdRisposteDettaglioAttributo != 0)
                            && (SelectedAttributo4 != null && SelectedAttributo4.IdAttributo != 0)
                            && (SelectedAttributo5 != null && SelectedAttributo5.IdRisposteDettaglioAttributo != 0);
                        break;
                    default:
                        break;
                }

                if (NrAttrObbligatori > 0)
                    AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionAttributiN.Replace("NNN", NrAttrObbligatori.ToString());
                else
                    AlertModel.Description = AppResources.StringMissingFieldsFormDescriptionAttributi;
            }

            return flgContinue;
        }

        public void SvuotaForm()
        {
            MessaggioInserimentoOk = string.Empty;
            DettagliInseriti = null;

            if (IsVisibleStato)
                DettaglioAperto = Dettaglio?.FirstOrDefault();
            else
                DettaglioAperto = new AuditRisposteDettaglio();

            IsEnabled = true;
            SelectedListAttributo4 = new List<object>();

            InitText();
            InitCombobox();
            InitRisoluzione();

            MessagingCenter.Send(this, Messages.Reset.ToString());
        }

        private async Task UpdateDomanda()
        {
            await FunctionsHelper.UpdateStatoAudit(Audit, 3);

            AuditDomandeRisposte dom = Domanda;
            bool aggiornaDomanda = false;

            if (Dettaglio == null)
            {
                if (Stato.ID == 1)
                {
                    dom.Risposta = (dom.RisposteNcDft != null && dom.RisposteNcDft.Contains(string.Format("-{0}|", dom.Risposta))) ? "0" : dom.Risposta;
                    dom.DataRisposta = DateTime.Now;
                    dom.Sincronizzato = false;

                    aggiornaDomanda = true;
                }
            }
            else
            {
                if (Stato.ID == 3 || Stato.ID == 4)
                {
                    int contaNC = await App.Database.ContaNc(Audit.IdAudit, Domanda.IdRispostaGuid, new int[] { 3, 4 });

                    if (contaNC == 0 && TipoAudit.IdTipoAudit != 8 && (new[] { 1, 2, 9, 10, 11 }).Contains(dom.IdTipoDomanda))
                    {
                        if (Stato.ID == 3)
                            dom.Risposta = "1";

                        if (Stato.ID == 4)
                            dom.Risposta = "0";

                        aggiornaDomanda = true;
                    }
                }

                if (Stato.ID == 5 || Stato.ID == 6)
                {
                    int contaNC = await App.Database.ContaNc(Audit.IdAudit, Domanda.IdRispostaGuid, new int[] { 5, 6 });

                    if (contaNC == 0 && (new[] { 1, 2, 9, 10, 11 }).Contains(dom.IdTipoDomanda))
                    {
                        dom.Risposta = "1";
                        aggiornaDomanda = true;
                    }
                }
            }

            if (aggiornaDomanda)
            {
                await App.Database.UpdateAuditDomandeRisposteAsync(dom);
                await App.Database.UpdateSignatures(Domanda.IdRispostaGuid, false);
            }

            IsDirty = false;
        }

        private NavigationParameterModel GeneraParametro()
        {
            return new NavigationParameterModel
            {
                Audit = Audit,
                Checklist = Checklist,
                Categoria = Categoria,
                Domanda = Domanda,
                Dettaglio = DettagliInseriti,
                NavigationFrom = NavigationFrom.Dettaglio
            };
        }

        public void GeneraBottoniAllegati()
        {
            List<BottoneDomanda> listaBottoni = new List<BottoneDomanda>();

            if (AggiungiAllegatoNCVisible)
            {
                if (AggiungiAllegatoFSNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachFsCommand, TestoBottone = AppResources.StringAddFileSystem, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_caricaAllegato_dark.png") });
                if (AggiungiAllegatoBarcodeNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachBarcodeCommand, TestoBottone = AppResources.StringAddBarcode, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_caricaAllegato_dark.png") });
                if (AggiungiAllegatoFotoNCVisible)
                {
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachPhotoCommand, TestoBottone = AppResources.StringAddPhoto, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_fotoEdit_dark.png") });
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddGalleryCommand, TestoBottone = AppResources.StringAddGallery, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_gallery_dark.png") });
                }
                if (AggiungiAllegatoDisegnoNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachDrawingCommand, TestoBottone = AppResources.StringAddDrawing, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_disegno_dark.png") });
                if (AggiungiAllegatoVideoNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachVideoCommand, TestoBottone = AppResources.StringAddVideo, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_video_dark.png") });
                if (AggiungiAllegatoAudioNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachAudioCommand, TestoBottone = AppResources.StringAddAudio, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_audio_dark.png") });
                if (AggiungiAllegatoEvidenzaNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachEvidenzaCommand, TestoBottone = AppResources.StringAddEvidenza, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_evidenze_dark.png") });
                if (AggiungiAllegatoTabellaNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachTabellaCommand, TestoBottone = AppResources.StringAddTable, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_tabelle_dark.png") });
                if (AggiungiAllegatoDocumentiNCVisible)
                    listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachDocumentiCommand, TestoBottone = AppResources.StringAddDocs, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_documenti_dark.png") });
            }

            if (AggiungiAllegatoMappaVisible)
                listaBottoni.Add(new BottoneDomanda { IsButtonEnabled = IsEnabled, ButtonCommand = AddAttachMappaCommand, TestoBottone = AppResources.StringAddMaps, ButtonImage = FunctionsHelper.SetImageString("icon_azioni_mappatura_dark.png") });

            AllegatiDisponibili = new ObservableCollection<BottoneDomanda>(listaBottoni);
        }

        private async Task GoToPage(Type pagina, string parametroAzione, bool disegno = false, bool photo = false, bool gallery = false)
        {
            bool doAction;

            if (Dettaglio != null && Dettaglio.Count() > 0)
                doAction = await UpdateAction("F");
            else
                doAction = await InsertAction("F");

            if (doAction)
            {
                NavigationParameterModel parameterModel = GeneraParametro();
                parameterModel.TargetType = pagina;
                parameterModel.Titolo = GeneraTestoCompleto();

                if (pagina == typeof(AddPhotoPage))
                    parameterModel.Drawing = disegno;

                if (pagina == typeof(AddPhotoPage))
                    parameterModel.Photo = photo;

                if (pagina == typeof(AddPhotoPage))
                    parameterModel.Gallery = gallery;

                MessagingCenter.Send<object, NavigationParameterModel>(this, parametroAzione, parameterModel);
            }
            else
                MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }

        private async Task<bool> UpdateAction(string s)
        {
            IsEnabled = false;
            bool oldIsTextEnabled = IsTextEnabled;
            IsTextEnabled = false;

            int insertRows = 0;
            bool testForm = CheckFormUpdate();

            if (!testForm)
            {
                MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), AlertModel);
                IsEnabled = true;
                IsTextEnabled = oldIsTextEnabled;
            }
            else
            {
                AuditRisposteDettaglio dettaglioModificato = DettaglioAperto;
                dettaglioModificato.IdRisposteDettaglioAttributo1 = SelectedAttributo1?.IdRisposteDettaglioAttributo;
                dettaglioModificato.IdRisposteDettaglioAttributo2 = SelectedAttributo2?.IdRisposteDettaglioAttributo;
                dettaglioModificato.IdRisposteDettaglioAttributo3 = SelectedAttributo3?.IdRisposteDettaglioAttributo;
                dettaglioModificato.IdRisposteDettaglioAttributo4 = SelectedAttributo4?.IdAttributo;
                dettaglioModificato.IdRisposteDettaglioAttributo5 = SelectedAttributo5?.IdRisposteDettaglioAttributo;
                dettaglioModificato.IdStatoNC = Stato.ID;
                dettaglioModificato.IdUtenteModifica = Settings.UserId;
                dettaglioModificato.Nota = Nota;
                dettaglioModificato.Testo = (TipoAudit.NrCampiTestoNc == 1) ? TestoNc1 : $"{TestoNc1}|{TestoNc2}|{TestoNc3}|{TestoNc4}|";

                if (!string.IsNullOrEmpty(DataPrevistaRisoluzioneAsString))
                    dettaglioModificato.DataPrevistaRisoluzione = DateTime.Parse($"{DataPrevistaRisoluzioneAsDate[0].ToString()} {DataPrevistaRisoluzioneAsDate[1].ToString()} {DataPrevistaRisoluzioneAsDate[2].ToString()}");

                insertRows = App.Database.UpdateAuditRispostaDettaglio(dettaglioModificato);

                DettagliInseriti = new AuditRisposteDettaglio[] { dettaglioModificato };
            }

            if (insertRows > 0)
            {
                await UpdateDomanda();

                if (s.Equals("S"))
                {
                    if (Dettaglio == null)
                    {
                        ConfirmModel alert = new ConfirmModel
                        {
                            Title = AppResources.StringNuovoDettaglioOK,
                            Description = AppResources.StringNuovaNC,
                            OkButton = AppResources.LabelSi,
                            KoButton = AppResources.LabelNO,
                        };

                        MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), alert);
                    }
                    else
                        MessagingCenter.Send(this, Messages.GoBack.ToString(), true);
                }

                IsEnabled = true;
            }
            else
                MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);

            return insertRows > 0;
        }

        private async Task<bool> InsertAction(string s)
        {
            IsEnabled = false;
            IsTextEnabled = false;
            int insertRows = 0;
            bool testForm = CheckFormInsert();
            int i = 0;

            if (!testForm)
            {
                MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), AlertModel);
                IsEnabled = true;
                IsTextEnabled = true;
            }
            else
            {
                AuditRisposteDettaglio nuovoDettaglio = new AuditRisposteDettaglio()
                {
                    IdAudit = Audit.IdAudit,
                    IdRispostaGuid = Domanda.IdRispostaGuid,
                    IdRisposteDettaglioMacroAttributo1 = DettaglioAperto.IdRisposteDettaglioMacroAttributo1,
                    IdRisposteDettaglioMacroAttributo2 = DettaglioAperto.IdRisposteDettaglioMacroAttributo2,
                    IdRisposteDettaglioMacroAttributo3 = DettaglioAperto.IdRisposteDettaglioMacroAttributo3,
                    IdRisposteDettaglioMacroAttributo4 = DettaglioAperto.IdRisposteDettaglioMacroAttributo4,
                    IdRisposteDettaglioMacroAttributo5 = DettaglioAperto.IdRisposteDettaglioMacroAttributo5,
                    IdRisposteDettaglioAttributo1 = SelectedAttributo1?.IdRisposteDettaglioAttributo,
                    IdRisposteDettaglioAttributo2 = SelectedAttributo2?.IdRisposteDettaglioAttributo,
                    IdRisposteDettaglioAttributo3 = SelectedAttributo3?.IdRisposteDettaglioAttributo,
                    IdRisposteDettaglioAttributo4 = SelectedAttributo4?.IdAttributo,
                    IdRisposteDettaglioAttributo5 = SelectedAttributo5?.IdRisposteDettaglioAttributo,
                    IdStatoNC = Stato.ID,
                    IdUtenteModifica = Settings.UserId,
                    Nota = Nota,
                    Testo = (TipoAudit.NrCampiTestoNc == 1) ? TestoNc1 : $"{TestoNc1}|{TestoNc2}|{TestoNc3}|{TestoNc4}|",
                    NcMultipla = Guid.NewGuid()
                };

                if (!string.IsNullOrEmpty(DataPrevistaRisoluzioneAsString))
                    nuovoDettaglio.DataPrevistaRisoluzione = DateTime.Parse($"{DataPrevistaRisoluzioneAsDate[0].ToString()} {DataPrevistaRisoluzioneAsDate[1].ToString()} {DataPrevistaRisoluzioneAsDate[2].ToString()}");

                if (IsVisibleAttributo4)
                {
                    DettagliInseriti = new AuditRisposteDettaglio[1];
                    nuovoDettaglio.IdRispostaDettaglioGuid = Guid.NewGuid();

                    if (TipoAudit.TipoInCaricoA?.Equals("M") != true)
                    {
                        insertRows += await App.Database.InsertAuditRispostaDettaglio(nuovoDettaglio);

                        if (insertRows > 0)
                            DettagliInseriti[0] = nuovoDettaglio;
                    }
                    else
                    {
                        DettagliInseriti = new AuditRisposteDettaglio[SelectedListAttributo4.Count];

                        foreach (object item in SelectedListAttributo4)
                        {
                            nuovoDettaglio.IdRisposteDettaglioAttributo4 = (item as Attributo4).IdAttributo;
                            nuovoDettaglio.IdRispostaDettaglioGuid = Guid.NewGuid();

                            insertRows += await App.Database.InsertAuditRispostaDettaglio(nuovoDettaglio);

                            if (insertRows > 0)
                                DettagliInseriti[i] = nuovoDettaglio;

                            i++;
                        }
                    }
                }
                else
                {
                    DettagliInseriti = new AuditRisposteDettaglio[1];
                    nuovoDettaglio.IdRispostaDettaglioGuid = Guid.NewGuid();

                    insertRows += await App.Database.InsertAuditRispostaDettaglio(nuovoDettaglio);

                    if (insertRows > 0)
                        DettagliInseriti[0] = nuovoDettaglio;
                }
            }

            if (insertRows > 0)
            {
                await UpdateDomanda();

                if (s.Equals("S"))
                {
                    if (Dettaglio == null)
                    {
                        ConfirmModel alert = new ConfirmModel
                        {
                            Title = AppResources.StringNuovoDettaglioOK,
                            Description = AppResources.StringNuovaNC,
                            OkButton = AppResources.LabelSi,
                            KoButton = AppResources.LabelNO,
                        };

                        MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), alert);
                    }
                    else
                        MessagingCenter.Send(this, Messages.GoBack.ToString(), true);
                }
                else
                {
                    Dettaglio = DettagliInseriti;
                    DettaglioAperto = DettagliInseriti.FirstOrDefault();
                }

                IsEnabled = true;
            }
            else
                MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);

            IsTextEnabled = true;
            return insertRows > 0;
        }

        private string GeneraTestoCompleto()
        {
            string retValue = string.Empty;

            if (TipoAudit.NrCampiTestoNc > 1)
            {
                string[] tmpArray = DettaglioAperto.Testo.Split(new[] { "|" }, StringSplitOptions.RemoveEmptyEntries);
                retValue = tmpArray[0];
                //string[] placeholderTextArray = TipoAudit.PlaceholderTestoNc?.Split(new[] { "|" }, StringSplitOptions.RemoveEmptyEntries);
                //string[] testoArray = new string[4];

                //for (int i = 0; i < tmpArray.Length; i++)
                //    testoArray[i] = tmpArray[i];

                //switch (TipoAudit.NrCampiTestoNc)
                //{
                //    case 1:
                //        retValue = $"<strong>{placeholderTextArray[0]}</strong>: {testoArray[0]}";
                //        break;
                //    case 2:
                //        retValue = $"<strong>{placeholderTextArray[0]}</strong>: {testoArray[0]} - <strong>{placeholderTextArray[1]}</strong>: {testoArray[1]}";
                //        break;
                //    case 3:
                //        retValue = $"<strong>{placeholderTextArray[0]}</strong>: {testoArray[0]} - <strong>{placeholderTextArray[1]}</strong>: {testoArray[1]} - <strong>{placeholderTextArray[2]}</strong>: {testoArray[2]}";
                //        break;
                //    case 4:
                //        retValue = $"<strong>{placeholderTextArray[0]}</strong>: {testoArray[0]} - <strong>{placeholderTextArray[1]}</strong>: {testoArray[1]} - <strong>{placeholderTextArray[2]}</strong>: {testoArray[2]} - <strong>{placeholderTextArray[3]}</strong>: {testoArray[3]}";
                //        break;
                //    default:
                //        break;
                //}
            }
            else
                retValue = DettaglioAperto.Testo;

            //retValue = $"<p>{retValue}</p>";

            return retValue;
        }

        public void GestStati(int? attributo4)
        {
            List<StatiDomandeRisposteDettaglio> listaStati = App.Database.GetStatiNCFilteredByInCarico(Audit.IdAudit, attributo4, DettaglioAperto.IdStatoNC);
            ListaStati = new List<ComboboxItem>();

            foreach (StatiDomandeRisposteDettaglio item in listaStati)
            {
                ListaStati.Add(new ComboboxItem()
                {
                    ID = item.IdStatoNC,
                    Name = (App.CultureInfo.TwoLetterISOLanguageName == "it") ? item.DescStatoNC : item.DescStatoEng,
                    Colore = item.ColoreStatoNC
                });
            }

            Stato = ListaStati.Where(w => w.ID == DettaglioAperto.IdStatoNC).FirstOrDefault();
        }
        #endregion

        #region Commands
        private async Task ExecuteSaveDetailCommand(string s)
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteSaveDetailCommand), s);

            await InsertAction(s);
        }

        private async Task ExecuteUpdateDetailCommand(string s)
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteUpdateDetailCommand), s);

            await UpdateAction(s);
        }

        private async Task ExecuteCancellaDettaglioCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteCancellaDettaglioCommand), string.Empty);

            bool risposta = await App.ShowConfirmAlert(ConfermaCancellazione);

            if (risposta)
            {
                DettaglioAperto.Attivo = false;
                DettaglioAperto.DataCancellazione = DateTime.Now;
                DettaglioAperto.IdUtenteModifica = Settings.UserId;

                App.Database.UpdateAuditRispostaDettaglio(DettaglioAperto);

                AuditDomandeRisposte domandaParametro = Domanda;

                if (domandaParametro == null)
                    domandaParametro = App.Database.GetAuditDomandaByIdRisposta(Audit.IdAudit, DettaglioAperto.IdRispostaGuid);

                int contaNC = await App.Database.ContaNc(Audit.IdAudit, domandaParametro.IdRispostaGuid, new int[] { });

                if (contaNC == 0 && (new[] { 1, 2, 9, 10, 11 }).Contains(domandaParametro.IdTipoDomanda) && (!string.IsNullOrEmpty(domandaParametro.Risposta) && domandaParametro.Risposta.Equals("0")))
                {
                    domandaParametro.Risposta = "1";

                    await App.Database.UpdateAuditDomandeRisposteAsync(domandaParametro);
                    await App.Database.UpdateSignatures(domandaParametro.IdRispostaGuid, false);
                }

                DettaglioAperto = null;

                MessagingCenter.Send(this, Messages.GoBack.ToString(), true);
            }
        }

        private void ExecuteResetDetailCommand()
        {
            SvuotaForm();
        }

        private void ExecuteOpenPickerCommand()
        {
            MessagingCenter.Send(this, Messages.OpenPicker.ToString());
        }

        private void ExecuteDataPrevistaRisoluzioneCommand(ObservableCollection<object> obj)
        {
            DataPrevistaRisoluzioneAsString = AppResources.StringEntroData.Replace(Constants.DateTimeFormat, DateTime.Parse($"{obj[0].ToString()} {obj[1].ToString()} {obj[2].ToString()}").ToString(Constants.DateTimeFormat));

            if (Dettaglio != null && Dettaglio.Count() > 0)
                ListaAllegatiVisibile = CheckFormUpdate() && AllegatiDisponibili.Count() > 0;
            else
                ListaAllegatiVisibile = CheckFormInsert() && AllegatiDisponibili.Count() > 0;
        }

        private async Task ExecuteSyncCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteSyncCommand), string.Empty);

            await SyncDB();
            InitText();
            InitCombobox();
            InitRisoluzione();

            MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }
        #endregion

        #region Attach Commands
        private async Task ExecuteAddAttachFsCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachFsCommand), string.Empty);

            bool doAction;

            if (Dettaglio != null && Dettaglio.Count() > 0)
                doAction = await UpdateAction("F");
            else
                doAction = await InsertAction("F");

            if (doAction)
            {
                int insertedRows = await AttachHelper.SaveFileFromFilesystem(Audit.IdAudit, Domanda.IdRispostaGuid, DettagliInseriti);

                if (insertedRows > 0)
                {
                    await FunctionsHelper.UpdateStatoAudit(Audit, 3);
                    await App.Database.UpdateSignatures(Domanda.IdRispostaGuid, false);
                }

                MessagingCenter.Send(this, Messages.OperationFinished.ToString(), (insertedRows > 0));
                MessagingCenter.Send(this, Messages.GoBack.ToString(), (insertedRows > 0));
            }
        }

        private async Task ExecuteAddAttachBarcodeCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachBarcodeCommand), string.Empty);
            MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }

        private async Task ExecuteAddAttachMappaCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachMappaCommand), string.Empty);

            await GoToPage(typeof(AddMappaturaPage), NavigationMessage.AttachMappa.ToString());
        }

        private async Task ExecuteAddAttachDocumentiCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachDocumentiCommand), string.Empty);
            MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }

        private async Task ExecuteAddAttachTabellaCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachTabellaCommand), string.Empty);
            MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }

        private async Task ExecuteAddAttachEvidenzaCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachEvidenzaCommand), string.Empty);
            MessagingCenter.Send(this, Messages.OperationFinished.ToString(), true);
        }

        private async Task ExecuteAddAttachAudioCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachAudioCommand), string.Empty);

            await GoToPage(typeof(AddAudioPage), NavigationMessage.AttachAudio.ToString());
        }

        private async Task ExecuteAddAttachVideoCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachVideoCommand), string.Empty);

            await GoToPage(typeof(AddVideoPage), NavigationMessage.AttachVideo.ToString());
        }

        private async Task ExecuteAddAttachDrawingCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachDrawingCommand), string.Empty);

            await GoToPage(typeof(AddPhotoPage), NavigationMessage.AttachPhoto.ToString(), true, false, false);
        }

        private async Task ExecuteAddAttachPhotoCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddAttachPhotoCommand), string.Empty);

            await GoToPage(typeof(AddPhotoPage), NavigationMessage.AttachPhoto.ToString(), false, true, false);
        }

        private async Task ExecuteAddGalleryCommand()
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            await FunctionsHelper.Log(nameof(this.ExecuteAddGalleryCommand), string.Empty);

            await GoToPage(typeof(AddPhotoPage), NavigationMessage.AttachPhoto.ToString(), false, false, true);
        }
        #endregion

        #region List Attach Commands
        private async Task ExecuteOpenAttachCommand(string stringId)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteOpenAttachCommand), stringId.ToString());
            int id = int.Parse(stringId);

            if (id == 0)
            {
                if (IsEnabled)
                {
                    DettagliInseriti = new AuditRisposteDettaglio[] { DettaglioAperto };
                    MessagingCenter.Send(this, QuestionMessages.AggiungiAllegato.ToString());
                }
            }
            else
            {
                if (id == -1)
                {
                    if (IsEnabled)
                    {
                        DettagliInseriti = new AuditRisposteDettaglio[] { DettaglioAperto };
                        AddAttachMappaCommand.Execute(null);
                    }
                }
                else
                {
                    AuditAllegatiRisposta allegato = ListaAllegati.Where(w => w.PKID == id).FirstOrDefault();

                    if (!string.IsNullOrEmpty(allegato.PathAllegato))
                    {
                        if (!allegato.Scaricato && Settings.IsDeviceConnected == false)
                            return;

                        if (!allegato.Scaricato)
                        {
                            string urlDownload = FunctionsHelper.GetFileDownloadUrl(allegato.PathAllegato);
                            await Browser.OpenAsync(urlDownload, BrowserLaunchMode.SystemPreferred);
                            allegato.PathAllegato = await FunctionsHelper.DownloadFile(allegato);
                            GetImmagineAllegato(allegato);
                        }
                        else
                            DependencyService.Get<IDocumentViewer>().ShowDocumentFile(allegato.PathAllegato);
                    }
                }
            }
        }

        private async Task ExecuteDeleteAttachCommand(int id)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteDeleteAttachCommand), id.ToString());
            AuditAllegatiRisposta auditAllegatiRisposta = ListaAllegati.Where(w => w.PKID == id).FirstOrDefault();

            if (auditAllegatiRisposta != null)
            {
                bool cancella = await App.ShowConfirmAlert(ConfermaCancellazione);

                if (cancella)
                {
                    auditAllegatiRisposta.Attivo = false;
                    auditAllegatiRisposta.DataCancellazione = DateTime.Now;
                    int retRow = App.Database.UpdateAuditAllegatoRisposta(auditAllegatiRisposta);

                    await FunctionsHelper.UpdateStatoAudit(Audit, 3);
                    await App.Database.UpdateSignatures(Domanda.IdRispostaGuid, false);

                    GetAllegati();
                }
            }
        }
        #endregion
    }
}